package com.csc714.app;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.csc714.data.*;
import com.csc714.maps.WifiOverlay;
import com.csc714.server.UDPServer;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

import android.content.Context;
import android.graphics.Color;
import android.location.*;
import android.net.wifi.ScanResult;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.widget.LinearLayout;

/**
 * @author phil
 * @author sushmita
 */
public class Main extends MapActivity {
	
	private MapView mapView;
	private WifiUserHistory myHistory;
	
	private String myName = "Waldo";
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
	        
        try{ 
        	/*
        	 * code to write some user histories to disk
        	 * not needed if the .obj already exists
        	 */
        	/*
	        ArrayList<WifiUserHistory> dummyHists = com.csc714.server.UDPServer.createDummyData();
	        // serialize some WifiUserHistory objects and write them to disk
	        ObjectOutputStream oos1 = new ObjectOutputStream(openFileOutput("wifi-soc-net.obj",MODE_WORLD_READABLE));
	        oos1.writeObject(dummyHists);
	        oos1.close();
	        */
        	
	        // read user histories from disk
	        ObjectInputStream ois = new ObjectInputStream(openFileInput("wifi-soc-net.obj"));
	        ArrayList<WifiUserHistory> hists = (ArrayList<WifiUserHistory>) ois.readObject();
	        
	        // get this user's history
	        this.myHistory = WifiUserHistory.getUserHistory(myName, hists);
	        
	        // assert: - phone already has a recently acquired location
	        //           e.g. manually requested from google maps app
	        //         - gps is turned on in g1 settings
	        // FYI: Raleigh is approximately (35.778, -78.687)
	        LocationManager locationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
	        Location location = locationManager.getLastKnownLocation("gps");
	        /*Location location = new Location("gps");
	        location.setLatitude(35.778);
	        location.setLongitude(-78.687);*/
	        
	        // get the current wifi situation
	        
	        WifiManager wifi = (WifiManager) getSystemService(Context.WIFI_SERVICE);
	        List<ScanResult> scans = null;
	        if(wifi.isWifiEnabled()) {

	        	// Requests to Start a scan
	        	wifi.startScan();
	
		    	//Wait till the scan completes
	    		Thread.sleep(200);
		
	    		// Access to the currently configured networks stored on your device.
	    		scans = wifi.getScanResults();
	        	        
		        // use scans to build list of WifiNetworks
		        Iterator<ScanResult> itr = scans.iterator();
		        ArrayList<WifiNetwork> networks = new ArrayList<WifiNetwork>();
		        ScanResult result = null;
		        while(itr.hasNext()) {
		        	result = (ScanResult)itr.next();
		        	networks.add(new WifiNetwork(result.SSID,result.level)); 
		        }
		        
		        // construct a WifiData object
		        WifiData wifiData = new WifiData(location.getLatitude(),location.getLongitude(),networks);
		        
		        // add the WifiData to the user's history
		        this.myHistory.addWifiData(wifiData);
		        
		        //WifiUserHistory.updateUserHistory(this.myHistory, hists);
		        
		        
		        // send our updated history to the server
	        	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	            ObjectOutputStream oos = new ObjectOutputStream(baos);
	            oos.writeObject(this.myHistory);
	        	
	            InetAddress serverAddr = InetAddress.getByName("152.1.56.147");
	            DatagramSocket socket = new DatagramSocket(UDPServer.port);  
	            //socket.setSoTimeout(9000);
	
	            DatagramPacket packet = new DatagramPacket(baos.toByteArray(),
	            											baos.toByteArray().length,
	            											serverAddr,
	            											UDPServer.port);
	            socket.send(packet);
	            
	            // now the server is updated. get all the data from it
	            
	            //request all the server's WifiUserHistory objects
	            packet = new DatagramPacket(baos.toByteArray(),
											0,
											serverAddr,
											UDPServer.port);
	            socket.send(packet);
	            
	            //Receive the data from the server	                 
	            byte[] receiveData = new byte[1048576]; // 1 MB
	            packet = new DatagramPacket(receiveData, receiveData.length);
	            socket.receive(packet);	            
	           
	            ByteArrayInputStream bais = new ByteArrayInputStream(packet.getData());
		        ois = new ObjectInputStream(bais);
	            
		        hists = (ArrayList<WifiUserHistory>)ois.readObject();				     
	            
	            // write the updated histories to disk
	            ObjectOutputStream oos1 = new ObjectOutputStream(openFileOutput("wifi-soc-net.obj",MODE_WORLD_READABLE));
		        oos1.writeObject(hists);
		        oos1.close();
		        
	        } // isWifiEnabled
 
	        mapView = (MapView) findViewById(R.id.mapView);
	        LinearLayout zoomLayout = (LinearLayout)findViewById(R.id.zoom);  
	        View zoomView = mapView.getZoomControls(); 
	 
	        zoomLayout.addView(zoomView, 
	            new LinearLayout.LayoutParams(
	                LayoutParams.WRAP_CONTENT, 
	                LayoutParams.WRAP_CONTENT)); 
	        mapView.displayZoomControls(true);

	        // center the map on our current location
            GeoPoint p = new GeoPoint((int) (location.getLatitude() * 1000000), (int) (location.getLongitude() * 1000000));
            MapController mc = mapView.getController();
            mc.setCenter(p);
			mc.setZoom(10);
			
			List<Overlay> list = mapView.getOverlays();
			
			WifiOverlay wifiOverlay = null;
			
	        WifiData data = null; // the wifi data at one location
	        //ArrayList<WifiNetwork> nets = new ArrayList<WifiNetwork>();
	        //WifiNetwork network = null;
	        WifiUserHistory hist = null;
	        String colorString = "";
	        int count = 0;
	        
	        for (Iterator<WifiUserHistory> it1 = hists.iterator(); it1.hasNext();){
	        	
	            if(count % 4 == 0) {
	            	colorString = "green";
	            } else if(count % 4 == 1){
	            	colorString = "yellow";
	            } else if(count % 4 == 2){
	            	colorString = "blue";
	            } else if(count % 4 == 3) {
	            	colorString = "red";
	            }    
	            
	        	hist = it1.next();
	        	wifiOverlay = new WifiOverlay(location,Color.parseColor(colorString));
	        	
		        for (Iterator<WifiData> it2 =  hist.getHistory().iterator (); it2.hasNext (); ) {
					data = it2.next();
										
					
					// all the networks at a given location
					/*
					nets = data.getWifiNetworks();
					for (Iterator<WifiNetwork> it3 = networks.iterator(); it3.hasNext (); ) {
						network = it3.next();																							
					}
					*/
					
					wifiOverlay.setWifiLocation(data.getLatitude(), data.getLongitude(),
							hist.getName() + "," +  data.getBestSignal().getSSID() + "," +
							WifiManager.calculateSignalLevel( data.getBestSignal().getLevel(), 10 ));
					
				}
		        
		        list.add(wifiOverlay);
		        count++;
	        }
				        
    	} catch(Exception e){
    		// TODO: better exception handling
    		e.toString();
    	}        
    }
    

	@Override
	protected boolean isRouteDisplayed() {
		return false;
	}

}