package com.csc714.maps;


import java.util.HashMap;
import java.util.Iterator;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.RectF;
import android.location.Location;
import android.location.LocationManager;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.Projection;

/**
 * Overlay class used to draw circles on the locations
 * of each contact in the contact list, along with their 
 * name and a line connecting them to your current
 * position.
 * 
 * @author phil
 * @author sushmita
 * 
 * @author Reto Meier
 * Author of Professional Android Application Development
 * http://www.amazon.com/gp/product/0470344717?tag=interventione-20
 *
 */
public class WifiOverlay extends Overlay {

  private HashMap<String, Location> friendLocations;
  private Location location;
  private GeoPoint locationPoint;
  
  private Paint paint;
  private Paint backPaint;

  private static int markerRadius = 7;
  
  /**
   * Create a new FriendLocationOverlay to show your contact's locations on a map
   * @param _context Parent application context
   */
  public WifiOverlay(Location currentLocation, int color) {
    super();

    this.setLocation(currentLocation);
    this.initializeHashMap();    

    // Create the paint objects
    backPaint = new Paint();
    backPaint.setARGB(200, 200, 200, 200);
    backPaint.setAntiAlias(true);
    
    paint = new Paint();
    paint.setColor(color);
    paint.setAntiAlias(true);
    paint.setFakeBoldText(true);    
  }

  /** Get your current location */
	public Location getLocation() {
		return location;
	}
	
	public void initializeHashMap() {
		friendLocations = new HashMap<String, Location>();
	}
  /** Set your current location */
	public void setLocation(Location location) {
	  this.location = location;
		
      Double latitude = location.getLatitude()*1E6;
      Double longitude = location.getLongitude()*1E6;
    
      locationPoint = new GeoPoint(latitude.intValue(),longitude.intValue());      
	}  
    
	public void setWifiLocation(Location l , String msg)
	{
		friendLocations.put(msg, l);
	}
	
	public void setWifiLocation(double latitude, double longitude, String msg){
		Location location = new Location(LocationManager.GPS_PROVIDER);
		location.setLatitude(latitude);
		location.setLongitude(longitude);
		
		this.setWifiLocation(location, msg);
	}
	
	public Location getWifiLocation(String key) {
		return friendLocations.get(key);
	}
  
  @Override
  public void draw(Canvas canvas, MapView mapView, boolean shadow) {	  

    // Get the map projection to convert lat/long to screen coordinates
    Projection projection = mapView.getProjection();
    
    Point lPoint = new Point();
    projection.toPixels(locationPoint, lPoint);
    
    // Draw the overlay
    if (shadow == false) {
      if (friendLocations.size() > 0) {
        Iterator<String> e = friendLocations.keySet().iterator();
        do {
          // Get the name and location of each contact
          String name = e.next();          
          Location location = friendLocations.get(name);
          
          // Convert the lat / long to a Geopoint
          Double latitude = location.getLatitude()*1E6;
          Double longitude = location.getLongitude()*1E6;
          GeoPoint geopoint = new GeoPoint(latitude.intValue(),longitude.intValue());

            Point point = new Point();
            projection.toPixels(geopoint, point);            
            
            // Draw a marker at the contact's location.
            RectF oval = new RectF(point.x-markerRadius,
                                   point.y-markerRadius,
                                   point.x+markerRadius,
                                   point.y+markerRadius);
            
            canvas.drawOval(oval, backPaint);
            oval.inset(2, 2);
            canvas.drawOval(oval, paint);
            
            // Draw wifi text next to their position.
            float textWidth = paint.measureText(name);
            float textHeight = paint.getTextSize();
            RectF textRect = new RectF(point.x+markerRadius, point.y-textHeight,
                                       point.x+markerRadius+8+textWidth, point.y+4);
            canvas.drawRoundRect(textRect, 3, 3, backPaint);
            canvas.drawText(name, point.x+markerRadius+4, point.y, paint);

        } while (e.hasNext());
      }
    }
    super.draw(canvas, mapView, shadow);
  }
	  
	@Override
	public boolean onTap(GeoPoint point, MapView mapView) {
	  // Do not react to screen taps.
	  return false;
	}
}
