package com.csc714.server;

import com.csc714.data.*;

import java.util.ArrayList;
import java.util.Iterator;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;

public class UDPServer {

	public static void main(String[] args) {		
		
		System.out.println("UDP Server");
		
		try {
			
			DatagramSocket socket = new DatagramSocket(UDPServer.port);
			
			ByteArrayOutputStream baos = null;
			ObjectOutputStream oos = null;
			
			byte[] receiveData = new byte[1048576]; // 1 MB
			
			ArrayList<WifiUserHistory> history = UDPServer.createDummyData();
			WifiUserHistory user = null;
			
			System.out.println("Server contains:");			
			for (Iterator<WifiUserHistory> it1 = history.iterator (); it1.hasNext (); ) {
				user = it1.next();
				System.out.println( user.getName() + " (" + user.getHistory().size()+ ")");
			}
			
			while(true){
								
				DatagramPacket receivePacket = new DatagramPacket(receiveData, receiveData.length);
				
				System.out.println("\nWaiting for packet...");
				socket.receive(receivePacket);
				
				InetAddress addr = receivePacket.getAddress();
				
				System.out.println("Packet received, from " + addr.getHostAddress());
				
				if(receivePacket.getLength() > 0){
					// user has sent a WifiUserHistory object
					// add it to our history
					System.out.println("Processing packet of length: " + receivePacket.getLength());					          
		            
		            ByteArrayInputStream bais = new ByteArrayInputStream(receivePacket.getData());
			        ObjectInputStream ois = new ObjectInputStream(bais);
			        WifiUserHistory inHist = (WifiUserHistory) ois.readObject();
			        
			        WifiUserHistory.updateUserHistory(inHist, history);
			        
			        System.out.println("Added " + inHist.getName() + " to server");
					System.out.println("Server contains:");			
					for (Iterator<WifiUserHistory> it1 = history.iterator (); it1.hasNext (); ) {
						user = it1.next();
						System.out.println( user.getName() + " (" + user.getHistory().size()+ ")");
					}
				} else {
					// contains no data 
					// this means the user has made a request for data
					// send back an array of WifiUserHistory's
					System.out.println("Request Message received");
					
					System.out.println("Creating object stream...");
		        	baos = new ByteArrayOutputStream();
		            oos = new ObjectOutputStream(baos);
		            //oos.writeObject(history.toArray()); // send an array back
		            oos.writeObject(history);
		        	
		            System.out.println("Building packet...");
		            //socket = new DatagramSocket(); 
		            InetAddress clientAddr = InetAddress.getByName(addr.getHostAddress()); 
		            System.out.println(baos.toByteArray().length + " bytes");
		            DatagramPacket packet = new DatagramPacket(baos.toByteArray(),
		            											baos.toByteArray().length,
		            											clientAddr,
		            											UDPServer.port);
		            System.out.println("Sending history data to " + clientAddr.getHostAddress() + "...");
		            Thread.sleep(5000);
		            socket.send(packet);
				}
				
			}
			
		} catch (Exception e) {			
			e.printStackTrace();
		}
		
		
	}
	
	public static ArrayList<WifiUserHistory> createDummyData(){
		
		ArrayList<WifiUserHistory> result = new ArrayList<WifiUserHistory>();
		ArrayList<WifiNetwork> wifiNetworks = null;
		WifiData wifiData = null;
		ArrayList<WifiData> history = null;
		WifiUserHistory userHist = null;
		
		/*
		 * First user
		 */
        history = new ArrayList<WifiData>();
        
		wifiNetworks = new ArrayList<WifiNetwork>();
        wifiNetworks.add(new WifiNetwork("aaa",-10));
        wifiNetworks.add(new WifiNetwork("bbb",-20));
        
        wifiData = new WifiData(35.7,-78.6,wifiNetworks);
        history.add(wifiData);
        
		wifiNetworks = new ArrayList<WifiNetwork>();
        wifiNetworks.add(new WifiNetwork("ccc",-35));
        wifiNetworks.add(new WifiNetwork("ddd",-45));
        
        wifiData = new WifiData(35.6,-78.6,wifiNetworks);        
        history.add(wifiData);
        
        userHist = new WifiUserHistory("Percy",history);
        
        result.add(userHist);
        
        /*
         * Second user
         */
        history = new ArrayList<WifiData>();
        
		wifiNetworks = new ArrayList<WifiNetwork>();
        wifiNetworks.add(new WifiNetwork("eee",-89));
        wifiNetworks.add(new WifiNetwork("fff",-88));
        
        wifiData = new WifiData(35.7,-78.7,wifiNetworks);
        history.add(wifiData);
        
		wifiNetworks = new ArrayList<WifiNetwork>();
        wifiNetworks.add(new WifiNetwork("ggg",-70));
        wifiNetworks.add(new WifiNetwork("hhh",-80));
        wifiNetworks.add(new WifiNetwork("iii",-90));
        wifiNetworks.add(new WifiNetwork("jjj",-10));
        
        wifiData = new WifiData(35.5,-78.6,wifiNetworks);        
        history.add(wifiData);
        
		wifiNetworks = new ArrayList<WifiNetwork>();
        wifiNetworks.add(new WifiNetwork("kkk",-11));
        
        wifiData = new WifiData(35.7,-78.5,wifiNetworks);
        history.add(wifiData);
        
        userHist = new WifiUserHistory("Waldo",history);   
        
        result.add(userHist);
		
		return result;
	}
	
	public static final int port = 5076;

}
